/*
 * clock for root window.
 */

/*
 * - finish alarms set up
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <locale.h>
#include <string.h>
#include <strings.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include "config.h"
#include "options.h"
#include "rootapp.h"

#define EVENTMASK VisibilityChangeMask | ExposureMask

/* Global varriables */
static int end_program = FALSE;

static char      *dpyname    = NULL;
static char      *geometry   = NULL;
static char      *colorname  = NULL;
static int        h, m, s;
static int        radius = 5;


/* Command line options and config file entries */
static Option opttab[] = {
    {'h', NULL,      NULL, O_HELP,   NULL,       FALSE, "display help"},
    {'v', NULL,      NULL, O_VERS,   NULL,       FALSE, "display version"},
    {'d', "display", NULL, C_STRING, &dpyname,   FALSE, "display name"},
    {'g', "geom",    NULL, C_STRING, &geometry,  FALSE, "set geometry"},
    {'c', "color",   NULL, C_STRING, &colorname, FALSE, "text color"},
    {'r', "radius",  NULL, C_INT,    &radius,    FALSE, "leds radius"},
    {0,   NULL,      NULL, C_NONE,   NULL,       FALSE, NULL}
};



/****** UPDATING FUNCTIONS ******/

void update_now() {
    time_t     tnow;
    struct tm *now;

    time(&tnow);
    now = localtime(&tnow);
    h = now->tm_hour;
    m = now->tm_min;
    s = now->tm_sec;
}


void draw_bits(RootApp *app, int t, int r, int p) {
    int v = t;
    int b[4], i = 0;

    b[3] = 0; b[2] = 0; b[1] = 0; b[0] = 0;
    while (v != 0) {
        if (i == 4) {
            fprintf(stderr, "Error : tens are too big (%d)\n", t);
            exit(1);
        }
        b[i] = v % 2;
        v = v / 2;
        i++;
    }
    for (i = 0 ; i < 4 ; i++) {
        int x = p;
        int y = 3 * (r + r / 2) - i * (r + r / 2);
        if (b[i]) {
            XDrawArc(display, app->bgpixmap, gc, x, y, r, r, 0, 23040);
            XFillArc(display, app->bgpixmap, gc, x, y, r, r, 0, 23040);
        } else
            XDrawArc(display, app->bgpixmap, gc, x, y, r, r, 0, 23040);
    }
}


void draw_time(RootApp *app) {
    draw_bits(app, h / 10, radius, 0);
    draw_bits(app, h % 10, radius, radius + radius / 2);

    draw_bits(app, m / 10, radius, radius * 4);
    draw_bits(app, m % 10, radius, radius * 5 + radius / 2);

    draw_bits(app, s / 10, radius, radius * 8);
    draw_bits(app, s % 10, radius, radius * 9 + radius / 2);
}


void get_default_geometry(XSizeHints *size) {
    size->width = radius * 10 + radius / 2 + 1;
    size->height = 4 * (radius + radius / 2);

    size->x = 0;
    size->y = 0;
}


/****** PROCESSING FUNCTIONS ******/

void main_loop(RootApp *app) {
    XEvent event;
    int cont = TRUE;

    if (colorname) set_color(colorname);
    draw_time(app);
    display_app(app);
    while (cont) {
        nextevent_or_timeout(&event, 1000);
        update_now();
        clear_bg(app);
        draw_time(app);
        display_app(app);
        if (end_program) cont = FALSE;
    }
}


/****** SIGNALS ******/

void install_handler();


void signals_handler(int sig) {
    switch (sig) {
        case SIGINT:
        case SIGTERM:
        case SIGQUIT:
            end_program = TRUE;
            break;
    }
}


void install_handler() {
    signal(SIGTERM, signals_handler);
    signal(SIGQUIT, signals_handler);
    signal(SIGINT,  signals_handler);
}


void manage_signals() {
    struct sigaction sa;

    sa.sa_handler = SIG_IGN;
#ifdef SA_NOCLDWAIT
    sa.sa_flags = SA_NOCLDWAIT;
#else
    sa.sa_flags = 0;
#endif
    sigemptyset(&sa.sa_mask);
    sigaction(SIGCHLD, &sa, NULL);
    install_handler();
}


/****** MAIN ******/


void set_defaults() {
    XSizeHints geom;

    if (radius < 3) radius = 3;
    get_default_geometry(&geom);
    if (! geometry) {
        geometry = xmalloc(MAXSTRLEN + 1);
        snprintf(geometry, MAXSTRLEN, "%dx%d%+d%+d",
                geom.width, geom.height, geom.x, geom.y);
    } else if (strchr(geometry, 'x') == NULL) {
        char *newgeom = xmalloc(strlen(geometry) + 12);
        char *posx = "";
        if ((geometry[0] != '-') && (geometry[0] != '+')) posx = "+";
        snprintf(newgeom, strlen(geometry) + 11, "%dx%d%s%s",
                geom.width, geom.height, posx, geometry);
        FREE(geometry);
        geometry = newgeom;
    }
    if (! colorname) colorname = xstrdup("red");
}


int main(int argc, char *argv[]) {
    RootApp    *app;

    manage_signals();
    setlocale(LC_ALL, "");
    parse_argv(opttab, argc, argv);
    set_defaults();
    update_now();

    app = create_app(dpyname, EVENTMASK, geometry, NULL, NULL);
    main_loop(app);
    close_app(app);
    free_vars(opttab);

    return 0;
}
